/*
** Problem v1.0
**
** A WWW version of PTS
**
** Copyright (c) 17.3.94 by Andreas Ley <ley@rz.uni-karlsruhe.de>
**
** Permission to use, copy, modify, and distribute this software for any
** purpose and without fee is hereby granted, provided that the above
** copyright notice appears in all copies. This software is provided "as is"
** and without any express or implied warranties.
**
** This program has been tested on a HP9000/720 with HP-UX A.08.07
** In this environment, neither lint -u nor gcc -Wall produce any messages.
** If you encounter any errors or need to make any changes to port it
** to another platform, please contact me.
**
** Version history
**
** Version 1.0 - 17.3.94
**	Initial version
*/

char header[]="problem v1.0\n(c) 1994 by Andreas Ley\n";

/* General definitions */

#define	CONFIG	"/usr/lib/sendmail.cf"
#define	TZ	"MEZ-1MESZ"
#define	MASTER	"webmaster@rz.uni-karlsruhe.de"
#define	REFERENCE	"<A HREF=\"/~Andreas.Ley/\"><EM>Andreas Ley</EM></A>"

/* File locations, config files and extensions */

#define	ROOT	"/usr/local/etc/httpd/lib/problems"
#define	ACCESS	".access"
#define	PRIVACCESS	".privaccess"
#define	MAIL	".mail"
#define	INFO	".info"
#define	EXTREPORTERS	".rep"
#define	EXTSUMMARY	".sum"
#define	EXTDESCRIPTION	".dsc"
#define	EXTREMARKS	".rem"
#define	EXTSOLUTION	".sol"

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <regex.h>
#include "problem_german.h"

#ifdef NEEDSTRERROR
extern int	sys_nerr;
extern char	*sys_errlist[];
#define	strerror(errno)	((errno)>=sys_nerr?"Undefined error":sys_errlist[errno])
#endif /* NEEDSTRERROR */

#define	FALSE	0		/* This is the naked Truth */
#define	TRUE	1		/* and this is the Light */

#define	max(a,b)	((a)>(b)?(a):(b))
#define	abs(a)	((a)<0?-(a):(a))


/* Functions from util.c */

char *makeword(char *line, char stop);
char *fmakeword(FILE *f, char stop, int *len);
char x2c(char *what);
void unescape_url(char *url);
void plustospace(char *str);

/* Functions from wildmat.c */

extern int wildmat();

/* Global variables */

char	root[]=ROOT;
char	master[]=MASTER;
char	*script_name;
char	script_path[MAXPATHLEN];
char	*path_info;
char	*remote_user;
char	*remote_host;
int	priviledged;
char	*form_name[16],*form_val[16];

/* Prints header of a HTML-Request */
void show_header(char *subtitle, char *description)
{
	(void)printf("Content-type: text/html\n\n");
	(void)printf("<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">\n");
	(void)printf("<HTML>\n");
	(void)printf("<HEAD>\n");
	(void)printf("<TITLE>%s - %s</TITLE>\n",TITLE,subtitle);
	(void)printf("<LINK REV=\"made\" HREF=\"mailto:%s\">\n",MASTER);
	(void)printf("</HEAD>\n");
	(void)printf("<BODY>\n");
	if (priviledged)
		(void)printf("<H1>* %s</H1>\n",subtitle);
	else
		(void)printf("<H1>%s</H1>\n",subtitle);
	if (description) {
		(void)printf(description,master);
		(void)printf("<P>\n");
	}
}



/* Prints trailer of a HTML-Request */
void show_trailer(char *path)
{
	if (path) {
		(void)printf("%s %s <A HREF=\"%s\">%s</A>",BACK,TO_MAIN_MENU,script_name,MAIN_MENU);
		if (*path)
			(void)printf(" %s %s <A HREF=\"%s?select\">%s</A>",OR,TO_MENU_SELECT,script_name,MENU_SELECT);
		(void)printf(".\n");
	}
	(void)printf(DISCLAIMER,master);
	(void)printf("\n%s\n",REFERENCE);
	(void)printf("</BODY>\n");
	(void)printf("</HTML>\n");
}



/* Shows an error page, based on wrong user input */
void show_error(char *error)
{
	show_header(ERROR,error);
	show_trailer(NULL);
	exit(0);
}



/* Shows an error page, based on internal misfunction */
void show_fatal(char *error)
{
	show_header(ERROR,error);
	(void)printf(ERROR_FATAL,master);
	(void)printf("<P>\n");
	show_trailer(NULL);
	exit(0);
}



/* Displays a file */
int show_file(char *fname, char *title, char *env)
{
	char	buffer[BUFSIZ];
	FILE	*src;

	if ((src=fopen(fname,"r"))) {
		(void)printf("\n<H3>%s</H3>\n",title);
		if (env)
			(void)printf("<%s>",env);
		while(fgets(buffer,sizeof(buffer),src))
			(void)printf("%s",buffer);
		if (env)
			(void)printf("</%s><P>\n",env);
		(void)fclose(src);
		return(TRUE);
	}
	return(FALSE);
}

char *timetag(char *user, char *node)
{
	static char	buffer[BUFSIZ];
	int		len;
	time_t		tloc;

	(void)sprintf(buffer,"%s\t%s\t",user,node);
	len=strlen(buffer);
	(void)time(&tloc);
	(void)strftime(buffer+len,sizeof(buffer)-len,"%a %b %d %H:%M:%S %Z %Y",localtime(&tloc));
	return(buffer);
}



/* Prints the Type: line with appropriate anchors */
void show_type(int mode)
{
	char	*ptr,*nptr;

	for(ptr=path_info;(nptr=strchr(ptr,'/'));ptr=nptr+1) {
		if (ptr==path_info)
			(void)printf("%s:\n",TYPE);
		*nptr='\0';
		(void)printf("<A HREF=\"%s?select\">%s</A> /\n",script_path,ptr);
		*nptr='/';
	}
	if (ptr==path_info)
		(void)printf("%s:\n",TYPE);
	if (mode)
		(void)printf("<A HREF=\"%s?list\">%s</A><P>\n",script_path,*ptr?ptr:NOTYPE);
	else
		(void)printf("%s<P>\n",*ptr?ptr:NOTYPE);
}



int intcmp(const void *s1, const void *s2)
{
	return(*(int *)s1-*(int *)s2);
}



/* Shows a list of problem summaries with appropriate anchors */
int show_problems(char *headers)
{
	char		path[MAXPATHLEN],buffer[BUFSIZ];
	int		unsolved,max,cnt,entry[1024];
	DIR		*dir;
	struct dirent	*dp;
	struct stat	buf;
	FILE		*src;

	if ((dir=opendir("."))) {
		max=0;
		while ((dp=readdir(dir)))
			if (dp->d_name[0]!='.'&&strlen(dp->d_name)>4&&!strcmp(dp->d_name+(strlen(dp->d_name)-4),EXTSUMMARY))
				entry[max++]=atoi(dp->d_name);
		(void)closedir(dir);

		if (max) {
			qsort((void *)entry,max,sizeof(char *),intcmp);
			(void)printf("<OL>\n");
			for (cnt=0;cnt<max;cnt++) {
				(void)sprintf(path,"%d"EXTSUMMARY,entry[cnt]);
				src=fopen(path,"r");
				(void)fgets(buffer,sizeof(buffer),src);
				(void)fclose(src);
				(void)sprintf(path,"%d"EXTSOLUTION,entry[cnt]);
				unsolved=access(path,F_OK);
				if (unsolved&&headers&&*headers) {
					(void)printf("%s",headers);
					*headers='\0';
				}
				if(unsolved||!headers)
					(void)printf("<LI> <A HREF=\"%s?show+%d\">%s</A>%s\n",script_path,entry[cnt],buffer,unsolved?UNSOLVED:SOLVED);
			}
			(void)printf("</OL>\n");
		}
	}
	return(headers&&!*headers);
}



/* Function to check wether a directory has additional subdirectories */
int has_subdirs(char *path)
{
	char		fullpath[MAXPATHLEN],*ptr;
	DIR		*dir;
	struct dirent	*dp;
	struct stat	buf;

	(void)strcpy(fullpath,path);
	ptr=strchr(fullpath,'\0');
	*ptr++='/';
	if ((dir=opendir(path))) {
		while ((dp=readdir(dir)))
			if (dp->d_name[0]!='.') {
				(void)strcpy(ptr,dp->d_name);
				(void)stat(fullpath,&buf);
				if (S_ISDIR(buf.st_mode)) {
					(void)closedir(dir);
					return(TRUE);
				}
			}
		(void)closedir(dir);
	}
	return(FALSE);
}



int strptrcmp(const void *s1, const void *s2)
{
	return(strcmp(*(char **)s1,*(char **)s2));
}



/* Shows a list of directories with appropriate anchors */
void show_selection(int recursive)
{
	char		path[MAXPATHLEN],buffer[BUFSIZ],*entry[1024],*ptr;
	int		max,cnt,subdirs;
	DIR		*dir;
	struct dirent	*dp;
	struct stat	buf;
	FILE		*src;

	if ((dir=opendir("."))) {
		max=0;
		while ((dp=readdir(dir)))
			if (dp->d_name[0]!='.') {
				(void)stat(dp->d_name,&buf);
				if (S_ISDIR(buf.st_mode))
					entry[max++]=strdup(dp->d_name);
			}
		(void)closedir(dir);

		if (max) {
			qsort((void *)entry,max,sizeof(char *),strptrcmp);
			(void)printf("<UL>\n");
			for (cnt=0;cnt<max;cnt++) {
				(void)printf("<LI> <A HREF=\"%s/%s?%s\">%s</A>\n",script_path,entry[cnt],has_subdirs(entry[cnt])?abs(recursive)==1?"recursive":abs(recursive)==2?"showall":abs(recursive)==3?"unsolved":"select":"list",entry[cnt]);
				if (recursive) {
					ptr=strchr(script_path,'\0');
					*ptr='/';
					strcpy(ptr+1,entry[cnt]);
					chdir(entry[cnt]);
					show_selection(recursive>0?-recursive:recursive);
					if (abs(recursive)>1)
						(void)show_problems(NULL);
					chdir("..");
					*ptr='\0';
				}
			}
			if (recursive>=0)
				(void)printf("<LI> <A HREF=\"%s?list\">%s</A>\n",script_path,TYPE_MISC);
			if (recursive>1&&!*path_info)
				(void)show_problems(NULL);
			(void)printf("</UL>\n");
		}
	}
}



void parse_form()
{
	int	cl,cnt=0;

	for (cnt=0,cl=atoi(getenv("CONTENT_LENGTH"));cl&&(!feof(stdin));cnt++) {
		form_val[cnt]=fmakeword(stdin,'&',&cl);
		plustospace(form_val[cnt]);
		unescape_url(form_val[cnt]);
		form_name[cnt]=makeword(form_val[cnt],'=');
	}
	form_name[cnt]=NULL;
}



char *get_form(char *name,char *error)
{
	int	cnt;

	for(cnt=0;form_name[cnt];cnt++)
		if (!strcmp(name,form_name[cnt]))
			return(form_val[cnt]);
	show_error(error);
}



/* Returns the appropriate smtp mailhost */
char *mailhost()
{
	static char	buffer[BUFSIZ];
	char		*ptr;
	FILE		*src;

	if ((src=fopen(CONFIG,"r"))) {
		while(fgets(buffer,sizeof(buffer),src))
			if (!strncmp(buffer,"DS",2)) {
				ptr=strchr(buffer,'\0');
				if (ptr[-1]=='\n')
					ptr[-1]='\0';
				(void)fclose(src);
				return(buffer+2);
			}
		(void)fclose(src);
	}
	return(NULL);
}



/* Checks if user is a valid email address */
int vrfy(char *user)
{
	static char	buffer[BUFSIZ];
	char		*host;
	int		sock,len;

	if (!(host=mailhost())) {
		sprintf(buffer,ERROR_CONFIG,CONFIG);
		show_fatal(buffer);
	}
	if ((sock=get_tcp_conn(host,"25"))<0) {
		sprintf(buffer,ERROR_CONNECT,host);
		show_fatal(buffer);
	}
	if (!(len=read(sock,buffer,sizeof(buffer)))) {
		(void)close(sock);
		show_fatal(ERROR_READ);
	}
	if (buffer[0]!='2') {
		(void)close(sock);
		show_fatal(ERROR_MESSAGE);
	}
	sprintf(buffer,"VRFY %s\r\n",user);
	if (len!=write(sock,buffer,len)) {
		(void)close(sock);
		show_fatal(ERROR_WRITE);
	}
	if (!(len=read(sock,buffer,sizeof(buffer)))) {
		(void)close(sock);
		show_fatal(ERROR_READ);
	}
	if (buffer[0]!='2') {
		(void)close(sock);
		return(FALSE);
	}
	(void)close(sock);
	return(TRUE);
}



/* Displays the main menu */
void main_menu()
{
	show_header(MAIN_MENU,MAIN_MENU_DESC);
	(void)printf("<DL>\n");
	(void)printf("<DT><A HREF=\"%s?select\">%s</A>\n",script_name,MENU_SELECT);
	(void)printf("<DD>%s\n",MAIN_MENU_SELECT);
	(void)printf("<DT><A HREF=\"%s?recursive\">%s</A>\n",script_name,MENU_RECURSIVE);
	(void)printf("<DD>%s\n",MAIN_MENU_RECURSIVE);
	(void)printf("<DT><A HREF=\"%s?showall\">%s</A>\n",script_name,MENU_SHOWALL);
	(void)printf("<DD>%s\n",MAIN_MENU_SHOWALL);
	if (priviledged) {
		(void)printf("<DT><A HREF=\"%s?unsolved\">%s</A>\n",script_name,MENU_UNSOLVED);
		(void)printf("<DD>%s\n",MAIN_MENU_UNSOLVED);
	}
	(void)printf("</DL>\n");
	show_trailer(NULL);
}



/* Shows a selection menu */
void menu_select()
{
	if (*path_info) {
		show_header(MENU_SELECT,MENU_SELECT_SUBDESC);
		show_type(FALSE);
	}
	else
		show_header(MENU_SELECT,MENU_SELECT_DESC);
	show_selection(0);
	show_trailer(path_info);
}



/* Shows a recursive selection menu */
void menu_recursive()
{
	if (*path_info) {
		show_header(MENU_RECURSIVE,MENU_RECURSIVE_SUBDESC);
		show_type(FALSE);
	}
	else
		show_header(MENU_RECURSIVE,MENU_RECURSIVE_DESC);
	show_selection(1);
	show_trailer(path_info);
}



/* Shows a recursive selection menu */
void menu_showall()
{
	if (*path_info) {
		show_header(MENU_SHOWALL,MENU_SHOWALL_SUBDESC);
		show_type(FALSE);
	}
	else
		show_header(MENU_SHOWALL,MENU_SHOWALL_DESC);
	show_selection(2);
	show_trailer(path_info);
}



/* Shows a recursive selection menu */
void menu_unsolved()
{
	if (*path_info) {
		show_header(MENU_UNSOLVED,MENU_UNSOLVED_SUBDESC);
		show_type(FALSE);
	}
	else
		show_header(MENU_UNSOLVED,MENU_UNSOLVED_DESC);
	show_selection(3);
	show_trailer(path_info);
}



void menu_list()
{
	char		path[MAXPATHLEN],buffer[BUFSIZ];
	DIR		*dir;
	struct dirent	*dp;
	FILE		*src;

	show_header(MENU_LIST,MENU_LIST_DESC);
	show_type(FALSE);

	(void)show_problems(NULL);
	(void)sprintf(path,"\"%s?new\"",script_path);
	(void)printf(MENU_LIST_NEW,path);

	(void)show_file(INFO,"<P>",NULL);
	(void)printf("<P>\n");
	show_trailer(root);
}



void menu_show(int id)
{
	char	path[MAXPATHLEN];
	int	solved=FALSE;

	show_header(MENU_SHOW,MENU_SHOW_DESC);
	show_type(TRUE);

	(void)sprintf(path,"%d"EXTSUMMARY,id);
	(void)show_file(path,SUMMARY,"PRE");
	(void)sprintf(path,"%d"EXTREPORTERS,id);
	(void)show_file(path,REPORTERS,"PRE");
	(void)sprintf(path,"%d"EXTDESCRIPTION,id);
	(void)show_file(path,DESCRIPTION,"PRE");
	(void)sprintf(path,"%d"EXTREMARKS,id);
	(void)show_file(path,REMARK,"PRE");
	(void)sprintf(path,"%d"EXTSOLUTION,id);
	solved=show_file(path,SOLUTION,"PRE");

	if (!solved) {
		(void)sprintf(path,"\"%s?again+%d\"",script_path,id);
		(void)printf(MENU_SHOW_AGAIN,path);
	}
	if (priviledged&&!solved) {
		(void)sprintf(path,"\"%s?remark+%d\"",script_path,id);
		(void)printf(MENU_SHOW_REMARK,path);
		(void)sprintf(path,"\"%s?solve+%d\"",script_path,id);
		(void)printf(MENU_SHOW_SOLVE,path);
		(void)printf("<P>\n");
	}
	if (!solved)
		(void)printf("<P>\n");
	show_trailer(root);
}



void menu_again(int id)
{
	char	path[MAXPATHLEN];

	show_header(MENU_NEW,MENU_NEW_DESC);
	show_type(TRUE);

	(void)sprintf(path,"%d"EXTSUMMARY,id);
	(void)show_file(path,SUMMARY,"PRE");
	(void)sprintf(path,"%d"EXTREPORTERS,id);
	(void)show_file(path,REPORTERS,"PRE");

	(void)printf("<H3>%s</H3>\n",REPORT_AGAIN);
	(void)printf("<FORM ACTION=\"%s?again+%d\" METHOD=POST>\n",script_path,id);
	(void)printf("%s: <INPUT NAME=\"user\" VALUE=\"%s\" SIZE=10>\n",USER,remote_user);
	(void)printf("%s: <INPUT NAME=\"node\" VALUE=\"%s\" SIZE=10>\n",NODE,remote_host);
	(void)printf("%s: <INPUT TYPE=submit VALUE=\"%s\">\n",SUBMIT,DOSUBMIT);
	(void)printf("</FORM>\n");
	show_trailer(root);
}



void menu_remark(int id)
{
	char	path[MAXPATHLEN];

	show_header(MENU_REMARK,MENU_REMARK_DESC);
	show_type(TRUE);

	(void)sprintf(path,"%d"EXTSUMMARY,id);
	(void)show_file(path,SUMMARY,"PRE");
	(void)sprintf(path,"%d"EXTDESCRIPTION,id);
	(void)show_file(path,DESCRIPTION,"PRE");

	(void)printf("<H3>%s</H3>\n",REMARK);
	(void)printf("<FORM ACTION=\"%s?remark+%d\" METHOD=POST>\n",script_path,id);
	(void)printf("<INPUT NAME=\"newremark\" SIZE=62,10><P>\n");
	(void)printf("%s: <INPUT NAME=\"user\" VALUE=\"%s\" SIZE=10>\n",USER,remote_user);
	(void)printf("%s: <INPUT NAME=\"node\" VALUE=\"%s\" SIZE=10>\n",NODE,remote_host);
	(void)printf("%s: <INPUT TYPE=submit VALUE=\"%s\">\n",SUBMIT,DOSUBMIT);
	(void)printf("</FORM>\n");
	show_trailer(root);
}



void menu_solve(int id)
{
	char	path[MAXPATHLEN];

	show_header(MENU_SOLVE,MENU_SOLVE_DESC);
	show_type(TRUE);

	(void)sprintf(path,"%d"EXTSUMMARY,id);
	(void)show_file(path,SUMMARY,"PRE");
	(void)sprintf(path,"%d"EXTDESCRIPTION,id);
	(void)show_file(path,DESCRIPTION,"PRE");

	(void)printf("<H3>%s</H3>\n",SOLUTION);
	(void)printf("<FORM ACTION=\"%s?solve+%d\" METHOD=POST>\n",script_path,id);
	(void)printf("<INPUT NAME=\"solution\" SIZE=62,10><P>\n");
	(void)printf("%s: <INPUT NAME=\"user\" VALUE=\"%s\" SIZE=10>\n",USER,remote_user);
	(void)printf("%s: <INPUT NAME=\"node\" VALUE=\"%s\" SIZE=10>\n",NODE,remote_host);
	(void)printf("%s: <INPUT TYPE=submit VALUE=\"%s\">\n",SUBMIT,DOSUBMIT);
	(void)printf("</FORM>\n");
	show_trailer(root);
}



void menu_new()
{
	show_header(MENU_NEW,MENU_NEW_DESC);
	show_type(TRUE);

	(void)printf("<FORM ACTION=\"%s?new\" METHOD=POST>\n",script_path);
	(void)printf("%s<P>\n<INPUT NAME=\"summary\" SIZE=62><P>\n",MENU_NEW_SUMMARY);
	(void)printf("%s<P>\n<INPUT NAME=\"description\" SIZE=62,10><P>\n",MENU_NEW_DESCRIPTION);
	(void)printf("%s: <INPUT NAME=\"user\" VALUE=\"%s\" SIZE=10>\n",USER,remote_user);
	(void)printf("%s: <INPUT NAME=\"node\" VALUE=\"%s\" SIZE=10>\n",NODE,remote_host);
	(void)printf("%s: <INPUT TYPE=submit VALUE=\"%s\">\n",SUBMIT,DOSUBMIT);
	(void)printf("</FORM>\n");
	show_trailer(root);
}



void report(int id)
{
	char		*user,*node,*summary,*description,path[MAXPATHLEN],buffer[BUFSIZ];
	int		len;
	FILE		*src,*dest;
	DIR		*dir;
	struct dirent	*dp;

	parse_form();
	user=get_form("user",NO_USER);
	if (!vrfy(user))
		show_error(ERROR_USER);
	node=get_form("node",NO_NODE);

	if (id) {
		len=strlen(user);
		(void)sprintf(path,"%d"EXTREPORTERS,id);
		if ((src=fopen(path,"r"))) {
			while(fgets(buffer,sizeof(buffer),src))
				if (!strncmp(buffer,user,len)&&buffer[len]=='\t') {
					(void)fclose(src);
					show_error(ERROR_TWICE);
				}
			(void)fclose(src);
		}
	}
	else {
		summary=get_form("summary",NO_SUMMARY);
		description=get_form("description",NO_DESCRIPTION);
		if ((dir=opendir("."))) {
			while ((dp=readdir(dir)))
				if (dp->d_name[0]!='.')
					id=max(id,atoi(dp->d_name));
			(void)closedir(dir);
		}
		id++;

		(void)sprintf(path,"%d"EXTSUMMARY,id);
		if (!(dest=fopen(path,"w")))
			show_fatal(strerror(errno));
		(void)fprintf(dest,"%s\n",summary);
		(void)fclose(dest);

		(void)sprintf(path,"%d"EXTDESCRIPTION,id);
		if (!(dest=fopen(path,"w")))
			show_fatal(strerror(errno));
		(void)fprintf(dest,"%s\n",description);
		(void)fclose(dest);
	}

	(void)sprintf(path,"%d"EXTREPORTERS,id);
	if (!(dest=fopen(path,"a")))
		show_fatal(strerror(errno));
	(void)fprintf(dest,"%s\n",timetag(user,node));
	(void)fclose(dest);

	show_header(REPORT,REPORT_DESC);
	show_type(TRUE);
	(void)printf("%s<P>\n",REPORT_SUCCESS);
	show_trailer(root);
}



void remark(int id)
{
	char		*user,*node,*newremark,path[MAXPATHLEN];
	FILE		*dest;

	parse_form();
	user=get_form("user",NO_USER);
	node=get_form("node",NO_NODE);
	newremark=get_form("newremark",NO_REMARK);

	(void)sprintf(path,"%d"EXTREMARKS,id);
	if (!(dest=fopen(path,"a")))
		show_fatal(strerror(errno));
	(void)fprintf(dest,"%s\n",timetag(user,node));
	(void)fprintf(dest,"%s\n\n",newremark);
	(void)fclose(dest);

	show_header(REMARK,REMARK_DESC);
	show_type(TRUE);
	(void)printf("%s<P>\n",REMARK_SUCCESS);
	show_trailer(root);
}



void solve(int id)
{
	char		*user,*node,*solution,path[MAXPATHLEN];
	FILE		*dest;

	parse_form();
	user=get_form("user",NO_USER);
	node=get_form("node",NO_NODE);
	solution=get_form("solution",NO_SOLUTION);

	(void)sprintf(path,"%d"EXTSOLUTION,id);
	if (!(dest=fopen(path,"w")))
		show_fatal(strerror(errno));
	(void)fprintf(dest,"%s\n",timetag(user,node));
	(void)fprintf(dest,"%s\n",solution);
	(void)fclose(dest);

	show_header(SOLUTION,SOLVE_DESC);
	show_type(TRUE);
	(void)printf("%s<P>\n",SOLVE_SUCCESS);
	show_trailer(root);
}



void usage(char *image)
{
	(void)fprintf(stderr,"Usage: %s [-h] [-v]\n",image);
	exit(1);
}


int main(int argc, char *argv[])
{
	int		c;
	extern int	optind;
	char		name[MAXPATHLEN],buffer[BUFSIZ],*query_string,*ptr;
	FILE		*src;
	int		hasaccess;

	while ((c=getopt(argc,argv,"vh?")) != EOF)
		switch ((char)c) {
		case 'v':
			(void)fprintf(stderr,header);
			exit(0);
		case 'h':
			(void)fprintf(stderr,header);
		case '?':
			usage(argv[0]);
		}

	remote_host=getenv("REMOTE_HOST");
	(void)chdir(root);
	hasaccess=FALSE;
	(void)sprintf(name,ACCESS);
	if ((src=fopen(name,"r"))) {
		while(!hasaccess&&fgets(buffer,sizeof(buffer),src)) {
			ptr=strchr(buffer,'\0');
			if (ptr[-1]=='\n')
				ptr[-1]='\0';
			if (wildmat(remote_host,buffer))
				hasaccess=TRUE;
		}
		(void)fclose(src);
	}
	else
		show_fatal(root);
	if (!hasaccess) {
		(void)sprintf(buffer,NO_ACCESS,remote_host,master);
		show_error(buffer);
	}

	priviledged=FALSE;
	(void)sprintf(name,PRIVACCESS);
	if ((src=fopen(name,"r"))) {
		while(!priviledged&&fgets(buffer,sizeof(buffer),src)) {
			ptr=strchr(buffer,'\0');
			if (ptr[-1]=='\n')
				ptr[-1]='\0';
			if (!strcmp(buffer,remote_host))
				priviledged=TRUE;
		}
		(void)fclose(src);
	}

	(void)putenv("TZ=" TZ);
	script_name=getenv("SCRIPT_NAME");
	(void)strcpy(script_path,script_name);
	remote_user=getenv("REMOTE_USER");
	if ((ptr=strchr(remote_host,'.')))
		*ptr='\0';
	path_info=strchr(script_path,'\0');
	(void)strcpy(path_info,getenv("PATH_INFO"));
	if (*path_info=='/')
		path_info++;
	query_string=getenv("QUERY_STRING");

	(void)chdir(path_info);

	if (!strcmp(getenv("REQUEST_METHOD"),"GET"))
		if (!strcmp("select",argv[optind]))
			menu_select();
		else if (!strcmp("recursive",argv[optind]))
			menu_recursive();
		else if (!strcmp("showall",argv[optind]))
			menu_showall();
		else if (!strcmp("unsolved",argv[optind]))
			menu_unsolved();
		else if (!strcmp("list",argv[optind]))
			menu_list();
		else if (!strcmp("show",argv[optind]))
			menu_show(atoi(argv[optind+1]));
		else if (!strcmp("again",argv[optind]))
			menu_again(atoi(argv[optind+1]));
		else if (priviledged&&!strcmp("remark",argv[optind]))
			menu_remark(atoi(argv[optind+1]));
		else if (priviledged&&!strcmp("solve",argv[optind]))
			menu_solve(atoi(argv[optind+1]));
		else if (!strcmp("new",argv[optind]))
			menu_new();
		else
			main_menu();
	else if (!strcmp(getenv("REQUEST_METHOD"),"POST"))
    if (!strcmp(getenv("CONTENT_TYPE"),"application/x-www-form-urlencoded"))
			if (!strncmp("again",query_string,5))
				report(atoi(argv[optind+1]));	
			else if (!strncmp("remark",query_string,5))
				remark(atoi(argv[optind+1]));	
			else if (!strncmp("solve",query_string,5))
				solve(atoi(argv[optind+1]));	
			else if (!strcmp("new",query_string))
				report(0);	
			else
				main_menu();
		else
			show_fatal(ERROR_CONTENT_TYPE);
	else
		show_fatal(ERROR_REQUEST_METHOD);

	return(0);
}
